from django.conf import settings
try:
    from urllib import parse as urllib_parse
except ImportError:  # Python 2
    import urllib as urllib_parse
    import urlparse
    urllib_parse.urlparse = urlparse.urlparse


class PermissiveCSRFMiddleware(object):
    def process_request(self, request):
        # Only process in DEBUG
        if not settings.DEBUG:
            return

        ref = request.META.get('HTTP_REFERER')
        if ref is None or ref == '':
            return

        ref_parts = urllib_parse.urlparse(ref)
        if request.is_secure() and ref_parts.scheme == 'http':
            # CsrfViewMiddleware performs the following check through django.util.http.same_origin(ref, 'http://%s/' % request.get_host())
            # which fails when http://example.com/ posts to https://example.com/accounts/login
            # In our case we'll only check the hostname part
            dest = 'https://%s' % request.get_host()
            dest_parts = urllib_parse.urlparse(dest)
            same_origin = (ref_parts.hostname, ref_parts.port) == (dest_parts.hostname, dest_parts.port)
            # print 'ref=%s:%s dest=%s:%s ? %s' % (ref_parts.hostname, ref_parts.port, dest_parts.hostname, dest_parts.port, same_origin)
            if same_origin:
                setattr(request, '_dont_enforce_csrf_checks', True)



from django.conf import settings
from django.test import TestCase
from django.test.client import RequestFactory
from permissivecsrf.middleware import PermissiveCSRFMiddleware as _PermissiveCSRFMiddleware


class PermissiveCSRFMiddleware(TestCase):

    def setUp(self):
        self.factory = RequestFactory()

    def tearDown(self):
        del self.factory

    def test_sets_flag_on_same_origin(self):
        settings.DEBUG = True
        factory = RequestFactory(**{'wsgi.url_scheme': 'https'})
        request = factory.get('/foo/')
        self.assertTrue(request.is_secure())

        referer = 'http://%s/' % request.get_host()
        request.META['HTTP_REFERER'] = referer
        self.assertTrue('HTTP_REFERER' in request.META)

        middleware = _PermissiveCSRFMiddleware()
        response = middleware.process_request(request)

        self.assertIsNone(response)
        self.assertTrue(hasattr(request, '_dont_enforce_csrf_checks'))
        self.assertTrue(getattr(request, '_dont_enforce_csrf_checks'))

    def _flag_not_set(self, request):
        middleware = _PermissiveCSRFMiddleware()
        response = middleware.process_request(request)

        self.assertIsNone(response)
        self.assertFalse(hasattr(request, '_dont_enforce_csrf_checks'))

    def test_does_not_set_flag_when_debug_is_false(self):
        settings.DEBUG = False
        request = self.factory.get('/foo/')
        self.assertTrue(request.build_absolute_uri().startswith('http://'))
        self.assertFalse('HTTP_REFERER' in request.META)

        self._flag_not_set(request)

    def test_does_not_set_flat_when_missing_http_referer(self):
        settings.DEBUG = True
        request = self.factory.get('/foo/')
        self.assertTrue(request.build_absolute_uri().startswith('http://'))
        self.assertFalse('HTTP_REFERER' in request.META)

        self._flag_not_set(request)

    def test_does_not_set_flag_if_destination_is_not_secure(self):
        settings.DEBUG = True
        request = self.factory.get('/foo/')
        request.META['HTTP_REFERER'] = 'http://example.com/'
        self.assertTrue(request.build_absolute_uri().startswith('http://'))
        self.assertTrue('HTTP_REFERER' in request.META)

        self._flag_not_set(request)

    def test_does_not_set_flag_if_origin_is_not_http_even_if_secure(self):
        settings.DEBUG = True
        factory = RequestFactory(**{'wsgi.url_scheme': 'https'})
        request = factory.get('/foo/')
        self.assertTrue(request.is_secure())

        request.META['HTTP_REFERER'] = 'https://example.com/'
        self.assertTrue('HTTP_REFERER' in request.META)

        self._flag_not_set(request)

    def test_does_not_set_flag_on_diff_origins(self):
        settings.DEBUG = True
        factory = RequestFactory(**{'wsgi.url_scheme': 'https'})
        request = factory.get('/foo/')
        self.assertTrue(request.is_secure())

        referer = 'http://foo.%s/' % request.get_host()
        request.META['HTTP_REFERER'] = referer
        self.assertTrue('HTTP_REFERER' in request.META)

        self._flag_not_set(request)




import os
from setuptools import setup


try:
    f = open(os.path.join(os.path.dirname(__file__), 'README.rst'), 'rt')
    long_description = f.read().strip()
    f.close()
except IOError:
    long_description = None

setup(
    # Basic package information
    name='django-permissivecsrf',
    version='1.0.0',
    packages=('permissivecsrf',),

    # test_suite='tests.runtests.main',

    # Packaging options
    include_package_data=True,
    install_requires = ['Django>=1.0'],

    # Metadata for PyPI
    url="https://github.com/philipmat/django-permissivecsrf",
    description='More permissive CSRF check for Django when moving between HTTP and HTTPS',
    long_description=long_description,
    author='Philip Mateescu',
    author_email='dev@philipm.at',
    license='BSD',
    keywords='django CSRF'.split(),
    platforms='any',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Utilities',
    ],
)

